<?php defined('SYSPATH') || die('No direct script access.');

class Controller_Teacher_Index extends Controller_Environment_Teacher
{
    public function action_index() {
        $teacher = Model_Teacher::with($this->user->TeacherID);
        $disciplines = $teacher->getDisciplines(User::instance()->SemesterID);
        $subjID = $discID = $gradeNum = $i = 0;
        $subjects = $teachers = $groups = $students = [];

        foreach ($disciplines as $dis) {
            // group disciplines by subjectID & gradeID
            if ($subjID != $dis->SubjectID || $gradeNum != $dis->GradeID) {
                $subjID = $dis->SubjectID;
                $gradeNum = $dis->GradeID;

                $subjects[++$i] = [
                    'SubjectName' => $dis->SubjectName,
                    'GradeNum'    => $dis->GradeNum,
                    'Degree'      => $dis->Degree,
                ];
            }

            if ($discID != $dis->ID) {
                $discID = $dis->ID;
                $subjects[$i]['Disciplines'][] = $dis;
            }

            $scientific = $dis->Subtype == Model_CourseWork::SCIENTIFIC;

            // todo: Pavel will write an optimized query
            if (!$scientific && !$teachers[$dis->ID]) {
                $list = $dis->getTeachers()->groupByUniqueKey('ID');
                $teachers[$dis->ID] = $this->getShortListOfTeachers($list, $dis->AuthorID, $this->user->TeacherID);
            }
            if (isset($dis->GroupNum) && $dis->GroupNum) {
                $groupDesc = $dis->GroupNum . ' гр';
                $groups[$dis->ID][] = $groupDesc;
            } else {
                $dis_groups = $dis->getAllGroups()->groupByUniqueKey('ID');
                foreach ($dis_groups as $dis_group) {
                    $groupDesc = '';
                    if ($dis_group['Degree'] === 'bachelor') {
                        $groupDesc = $dis->GroupNum . ' гр. бакалавриат' ;
                    }
                    if ($dis_group['Degree'] === 'master') {
                        $groupDesc = $dis->GroupNum . ' гр. магистратура ';
                    }
                    if ($dis_group['Degree'] === 'specialist') {
                        $groupDesc = $dis->GroupNum . ' гр. специалитет ';
                    }
                    if ($dis_group['Degree'] === 'postgraduate') {
                        $groupDesc = $dis->GroupNum . ' гр. аспирантура ';
                    }

                    if ($dis_group['FormID'] === 'очно-заочная') {
                        $groupDesc = $groupDesc.' ВО';
                    }
                    if ($dis_group['FormID'] === 'заочная') {
                        $groupDesc = $groupDesc.' ЗО';
                    }
                    $groups[$dis->ID][] = $dis_group['GradeNum'] . ' к. '. $dis_group['GroupNum'] . $groupDesc;
                }
            }


            if (!isset($students[$dis->ID])) {
                $students[$dis->ID] = array_filter($dis->getStudents(), function ($student) {
                    return $student->AttachType !== 'detach';
                });
            }
        }

        $session = Session::instance();
        $EMailChanged = $session['EMailChanged'];
        $session->set('EMailChanged', false);

        $this->twig->set([
            'Subjects'     => $subjects,
            'Teachers'     => $teachers,
            'Groups'       => $groups,
            // http://gitlab.mmcs.sfedu.ru/it-lab/grade/issues/50
            // Передача списка прикрепленных студентов отключена чтобы
            // не загромождать пользовательский интерфейс
            // и потому что списки студентов определяются только данными из 1С
            // теперь в списке дисциплин нельзя быстро увидеть список студентов
            // но можно вернуть эту функциональность, раскомментировав строку
            // 'Students'     => $students,
            'DisciplineCreationISAllowed'  => Model_System::loadConfig()->Functional->DisciplineCreation,
            'EMailChanged' => $EMailChanged,
        ])->set_filename('teacher/index');
    }

    public static function getShortListOfTeachers(&$teachers, $authorID = null, $vipID = null) {
        $short = [];

        // place vip & author at the beginning
        foreach ($teachers as $id => &$teacher) {
            if ($id == $authorID || $id == $vipID)
                $short += [$id => $teacher];
        }

        return $short + Arr::shuffle_assoc($teachers);
    }

    public function action_settings() {
        $this->twig->set_filename('settings');
    }
}
